# == Schema Information
#
# Table name: chunk_files
#
#  id                                   :bigint           not null, primary key
#  chunk_number(文件块数，从1开始)      :integer
#  chunk_size(文件块的大小)             :integer
#  current_chunk_size(当前文件块的大小) :integer
#  total_size(文件总大小)               :integer
#  identifier(文件标识)                 :string(255)
#  filename(文件名)                     :string(255)
#  total_chunks(文件块的总个数)         :integer
#  created_at                           :datetime         not null
#  updated_at                           :datetime         not null
#
module Whaleback
  class ChunkFile < ApplicationRecord
    LocalPath = './public/chunk_file'.freeze
    attr_accessor :file, :creator, :media
    
    after_save :write_file
    after_destroy :destroy_file

    def folder
      File.join(ChunkFile::LocalPath, identifier)
    end

    def file_path
      File.join(folder, filename)
    end

    def chunk_file_path
      File.join(folder, "#{filename}.part#{chunk_number}")
    end

    def uploading_file_path
      File.join(folder, "#{filename}.part#{chunk_number}.loading")
    end

    def all_finish?
      finish_indexs = Dir["#{folder}/*.part?"].map { |path| /.part(\d+)/.match(path)[1].to_i }.sort
      finish_indexs == (1..total_chunks).to_a
    end

    def finish_loading
      File.rename(uploading_file_path, chunk_file_path)
    end

    private

    def write_file
      FileUtils.mkdir_p folder
      File.open(uploading_file_path, 'wb') do |file|
        file.write(@file&.read)
      end
      finish_loading
      merge_file if all_finish?
    end

    def merge_file
      FileUtils.remove file_path, force: true
      File.open(file_path, 'ab') do |file|
        (1..chunk_number).each do |index|
          file.write File.open("#{file_path}.part#{index}", 'rb').read
        end
      end
      save_media
      FileUtils.remove_dir(folder, force: true)
    end

    def save_media
      mime_type = Rack::Mime.mime_type(File.extname(file_path))
      @media = assign_media_class(mime_type).create!(
        file: File.open(file_path, 'rb'),
        creator: @creator
      )
    end

    # 可覆盖，Obj class 继承有 STI type 问题，也可以细化分类
    def assign_media_class mime_type
      case mime_type
      when /^image/
        Media::Obj::Image
      when /^video/
        Media::Obj::Video
      else
        Media::Obj::Document
      end
    end

    def destroy_file
      FileUtils.rm([file_path, chunk_file_path, uploading_file_path], force: true)
    end
  end
end
